#version 330

uniform vec4 light_diffusecomponent;
uniform vec4 light_ambientcomponent;
uniform vec4 light_specularcomponent;

uniform vec4 material_ambient;
uniform vec4 material_diffuse;
uniform vec4 material_specular;

uniform vec3 cameraview;

uniform float specularcomponentfactor;

uniform sampler2D shadowmap;

in vec4 color_out;
in vec3 normal_out;
in vec4 vertex_out;
in vec3 lightpos;
in vec2 tex_coords;
in vec4 shadow_tex_coords;

out vec4 fragColor;


void main (void)  
{    
	vec3 N = normalize(normal_out);
	vec3 position = vertex_out.xyz / vertex_out.w;
	vec3 L = normalize(lightpos - position);


	vec4 ambientcomponent = light_ambientcomponent * material_ambient;
	float lamberfactor = max(0.0, dot(N, L));
	if(lamberfactor > 0.0)
	{	
		vec3 R = reflect(-L, N);
		vec3 E = normalize(cameraview);
		vec4 diffusecomponent = light_diffusecomponent * material_diffuse * lamberfactor;
		vec4 specularcomponent = light_specularcomponent * material_specular * pow(max(dot(R, E), 0.0), specularcomponentfactor);
		float att = 1.0f;

		//shadow mapping
		vec3 sm_tex_coords = shadow_tex_coords.xyz/shadow_tex_coords.w;
		sm_tex_coords.z -= 0.005;
		float depth = texture(shadowmap, sm_tex_coords.xy).r;
		float inShadow = (depth < sm_tex_coords.z) ? 0.2 : 1.0;

		fragColor = (((ambientcomponent + diffusecomponent) * color_out) + specularcomponent) * att * inShadow;
	}
	else
	{
		fragColor = ambientcomponent;
	}
}